package com.jeeplus.test.workContract.service.dto;

import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.utils.StringUtils;
import com.jeeplus.sys.utils.UserUtils;
import com.jeeplus.test.workContract.domain.WorkContractBorrow;
import com.jeeplus.test.workContract.domain.WorkContractFile;
import com.jeeplus.test.workContract.mapper.WorkContractBorrowMapper;
import com.jeeplus.test.workContract.mapper.WorkContractFileMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.UUID;

@Service
public class WorkContractBorrowService {

    @Resource
    private WorkContractBorrowMapper workContractBorrowMapper;

    public String saveInfo(WorkContractBorrowDto workContractBorrowDto) throws Exception {
        if (StringUtils.isNotEmpty(workContractBorrowDto.getId())) {
            WorkContractBorrow info = workContractBorrowMapper.selectFileByContractInfoId(workContractBorrowDto.getId());
            if (info != null) {
                return update(workContractBorrowDto, info.getId());
            }
        }
        return add(workContractBorrowDto);
    }

    /**
     * 合同登记新增
     */
    @Transactional(rollbackFor = Exception.class)
    public String add(WorkContractBorrowDto workContractBorrowDto) throws Exception{
        //获取当前登录人信息
        UserDTO userDTO = UserUtils.getCurrentUserDTO();
        String id = UUID.randomUUID().toString().replace("-", "");
        WorkContractBorrow borrow = new WorkContractBorrow();
        BeanUtils.copyProperties(workContractBorrowDto, borrow);
        borrow.setId(id);
        borrow.setCreateBy(userDTO.getId());
        borrow.setCreateDate(new Date());
        borrow.setUpdateBy(userDTO.getId());
        borrow.setUpdateDate(new Date());
        workContractBorrowMapper.insert(borrow);
        return id;
    }

    /**
     * 合同登记修改
     */
    @Transactional(rollbackFor = Exception.class)
    public String update(WorkContractBorrowDto workContractBorrowDto, String id) {
        //获取当前登录人信息
        UserDTO userDTO = UserUtils.getCurrentUserDTO();
        WorkContractBorrow borrow = new WorkContractBorrow();
        BeanUtils.copyProperties(workContractBorrowDto, borrow);
        borrow.setId(id);
        borrow.setUpdateBy(userDTO.getId());
        borrow.setUpdateDate(new Date());
        workContractBorrowMapper.updateById(borrow);
        return borrow.getContractInfoId();
    }

    /**
     * 根据id修改status
     */
    public void updateStatusById(WorkContractBorrowDto workContractBorrowDto) {
        workContractBorrowMapper.updateStatusById(workContractBorrowDto.getId(), workContractBorrowDto.getBorrowType());
    }

    public WorkContractBorrow findById(String id) {
        return workContractBorrowMapper.selectById(id);
    }

    public WorkContractBorrow findByContractInfoId(String id) {
        return workContractBorrowMapper.selectFileByContractInfoId(id);
    }
}
